if(typeof errorHandler != 'function'){
   window.errorHandler = function(error){
      console.log(error);
   };
}

function getCurrentEntityID() {
    var entityID = null;

    // Try to find entity ID element for Advanced (Multistep) Form
    var entityIDElement = $("#EntityFormView_EntityID");

    // If not found, try to find for Basic form
    if (entityIDElement.length === 0) {
        entityIDElement = $("input[id^='EntityFormControl_'][id$='_EntityFormView_EntityID']");
    }

    // If either has been found, set the entity ID to its value
    if (entityIDElement.length > 0) {
        entityID = entityIDElement.val();
    }
    return entityID;
};

function generateRequiredFieldValidatorId(fieldName) {
    return "RequiredFieldValidator" + fieldName;
}

function makeRequired(fieldName) {
 
	"use strict";
    try {
        if ($("#" + fieldName) != undefined) {

            var requiredFieldValidatorId = generateRequiredFieldValidatorId(fieldName);

            // If a required field validator already exists, don't do anything further
            for (var i = 0; i < Page_Validators.length; i++) {
                if (Page_Validators[i].id == requiredFieldValidatorId) {
                    return;
                }
            }

            $("#" + fieldName).prop('required', true);
            $("#" + fieldName).closest(".control").prev().addClass("required");
            // Create new validator
            var reqValidator = document.createElement('span');
            reqValidator.style.display = "none";
            reqValidator.id = requiredFieldValidatorId;
            reqValidator.controltovalidate = fieldName;
            reqValidator.errormessage = "<a href='#" + fieldName + "_label'>" + $("#" + fieldName + "_label").html() + " is a required field.</a>";
            reqValidator.initialvalue = "";
            reqValidator.evaluationfunction = function () {
                var value = null;
                var isOptionSetControl = ($("#" + fieldName).find("input[type='radio']").length > 0);

                if (isOptionSetControl) {
                    value = $("#" + fieldName).find("input[type='radio']:checked").val();
                }
                else {
                    value = $("#" + fieldName).val();
                } 

                // Special handling for masked input fields
                 if (value != null) {
                    value = value.replaceAll('"', '');
                }
                
                if (value == null || value == "") {
                    return false;
                } else {
                    return true;
                }
            };

            // Search for existing matching validator
            if (Page_Validators)
            // Add the new validator to the page validators array 
            Page_Validators.push(reqValidator);
        }
    }
    catch (error) {
        errorHandler(error);
    }
}
function makeNotRequired(fieldName) {

	"use strict";
	
    //Clean up previous validator <span> if present
    // if ($('#ValidationSummaryEntityFormView').length) {
    //     $('#ValidationSummaryEntityFormView').hide();
    // }
    try {
        if ($("#" + fieldName) != undefined) {
            var requiredFieldValidatorId = generateRequiredFieldValidatorId(fieldName);

            $("#" + fieldName).closest(".control").prev().removeClass("required");
            $("#" + fieldName).prop('required', false);
            for (var i = 0; i < Page_Validators.length; i++) {
                if (Page_Validators[i].id == requiredFieldValidatorId) {
                    Page_Validators.splice(i, 1);
                }
            }
        }
    }
    catch (error) {
        errorHandler(error);
    }
}
function OptionmakeRequired(fieldName) {
"use strict";
    try {
        if ($("#" + fieldName) != undefined) {
            $("#" + fieldName).prop('required', true);
            $("#" + fieldName).closest(".control").prev().addClass("required");
    
            var requiredFieldValidatorId = generateRequiredFieldValidatorId(fieldName);

            // If a required field validator already exists, don't do anything further
            for (var i = 0; i < Page_Validators.length; i++) {
                if (Page_Validators[i].id == requiredFieldValidatorId) {
                    return;
                }
            }

            // Create new validator
            var Requiredvalidator = document.createElement('span');
            Requiredvalidator.style.display = "none";
            Requiredvalidator.id = requiredFieldValidatorId;
            Requiredvalidator.controltovalidate = fieldName;
            Requiredvalidator.errormessage = "<a href='#" + fieldName + "_label'>" + $("#" + fieldName + "_label").html() + " is a required field.</a>";
            Requiredvalidator.initialvalue = "";
            Requiredvalidator.evaluationfunction = function () {
                var value = $("#" + fieldName).find("input[type='radio']:checked").val();
                if (value == null || value == "") {
                    return false;
                } else {
                    return true;
                }
            };

            // Add the new validator to the page validators array:
            Page_Validators.push(Requiredvalidator);
        }
    }
    catch (error) {
        errorHandler(error);
    }
}

function generateCustomFieldValidatorId(fieldName) {
    return fieldName + "CustomFieldValidator";
}

function addCustomValidator(fieldName, errorMessage, validationFunction, makeRequiredField, addFieldLabelToErrorMessage) {
 
	"use strict";
    try {
        if ($("#" + fieldName) != undefined) {

            var customFieldValidatorId = generateCustomFieldValidatorId(fieldName);

            // If the field validator already exists, don't do anything further
            for (var i = 0; i < Page_Validators.length; i++) {
                if (Page_Validators[i].id == customFieldValidatorId) {
                    return;
                }
            }

            if (makeRequiredField){
                $("#" + fieldName).prop('required', true);
                $("#" + fieldName).closest(".control").prev().addClass("required");
            }

            // Create new validator
            var varErrorMessage = "<a href='#" + fieldName + "_label'>";
            if (addFieldLabelToErrorMessage) {
                varErrorMessage += $("#" + fieldName + "_label").html() + ": ";
            }
            varErrorMessage += errorMessage + "</a>";

            var customValidator = document.createElement('span');
            customValidator.style.display = "none";
            customValidator.id = customFieldValidatorId;
            customValidator.controltovalidate = fieldName;
            customValidator.errormessage = varErrorMessage;
            customValidator.initialvalue = "";
            customValidator.evaluationfunction = function () {
                return validationFunction(fieldName);
            };

            // Search for existing matching validator
            if (Page_Validators)
                // Add the new validator to the page validators array 
                Page_Validators.push(customValidator);
        }
    }
    catch (error) {
        errorHandler(error);
    }
}

function removeCustomValidator(fieldName, isRequiredFieldValidator) {

	"use strict";
	
    //Clean up previous validator <span> if present
    // if ($('#ValidationSummaryEntityFormView').length) {
    //     $('#ValidationSummaryEntityFormView').hide();
    // }
    try {
        if ($("#" + fieldName) != undefined) {
            var requiredFieldValidatorId = generateCustomFieldValidatorId(fieldName);

            if (isRequiredFieldValidator){
                $("#" + fieldName).closest(".control").prev().removeClass("required");
                $("#" + fieldName).prop('required', false);
            }
            for (var i = 0; i < Page_Validators.length; i++) {
                if (Page_Validators[i].id == requiredFieldValidatorId) {
                    Page_Validators.splice(i, 1);
                }
            }
        }
    }
    catch (error) {
        errorHandler(error);
    }
}

function setFieldReadOnly(fieldName, readOnly, type) {
    try {
        type = type.toLowerCase();
 
        switch (type) {
            case "date":
            case "time":
            case "datetime":
                setDateTimeFieldReadOnly(fieldName, readOnly);
                break;
            case "lookup":
                setLookupFieldReadOnly(fieldName, readOnly);
                break;
            case "checkbox":
                setCheckboxFieldReadOnly(fieldName, readOnly);
                break;
            case "radio":
                setRadioFieldReadOnly(fieldName, readOnly);
                break;
            case "dropdown":
                setDropdownFieldReadOnly(fieldName, readOnly);
                break;
            default:
                if (!!readOnly) {
                    $("#" + fieldName).attr("readonly", true);

                    // Hide "dash" used when readonly text field contains no value
                    $("#" + fieldName).next("div.text-muted").ready(function() {
                        $("#" + fieldName).next("div.text-muted").hide();
                    });
                } else {
                    $("#" + fieldName).attr("readonly", false);
                }
                break;
        }
    }
    catch (err) {
        console.error("Error setFieldReadOnly: " + err.message);
        return;
    }
};

function setDateTimeFieldReadOnly(fieldName, readOnly) {
    if (readOnly) {
        $('#' + fieldName).siblings("div.datetimepicker").find("input, .input-group-addon").attr("readonly", true);
        $('#' + fieldName).siblings("div.datetimepicker").find("input, .input-group-addon").css("cursor", "not-allowed");
        $('#' + fieldName).siblings("div.datetimepicker").find("input, .input-group-addon").on("mousedown", function (e) { e.preventDefault(); return false; });
    } else {
        $('#' + fieldName).siblings("div.datetimepicker").find("input, .input-group-addon").attr("readonly", false);
        $('#' + fieldName).siblings("div.datetimepicker").find("input, .input-group-addon").css("cursor", "default");
        $('#' + fieldName).siblings("div.datetimepicker").find("input, .input-group-addon").off("mousedown");
    }
};

function setLookupFieldReadOnly(fieldName, readOnly) {
    if (readOnly) {
        $('#' + fieldName).siblings("div.input-group-btn").find("button").prop("disabled", true);
        $('#' + fieldName).siblings("div.input-group-btn").hide();
    } else {
        $('#' + fieldName).siblings("div.input-group-btn").find("button").prop("disabled", false);
        $('#' + fieldName).siblings("div.input-group-btn").show();
    }
};

function setCheckboxFieldReadOnly(fieldName, readOnly) {
    if (readOnly) {
        $('#' + fieldName).prop("disabled", true);
    } else {
        $('#' + fieldName).prop("disabled", false);
    }
};

function setRadioFieldReadOnly(fieldName, readOnly) {
    if (readOnly) {
        $('#' + fieldName).find("input[type='radio']").attr("disabled", true);
    } else {
        $('#' + fieldName).find("input[type='radio']").attr("disabled", false);
    }
};

function setDropdownFieldReadOnly(fieldName, readOnly) {
    if (readOnly) {
        $('#' + fieldName).attr("readonly", true);
        $('#' + fieldName).css("pointer-events", "none");
    } else {
        $('#' + fieldName).attr("readonly", false);
        $('#' + fieldName).css("pointer-events", "auto");
    }
};

// Returns checked value for the option set field
function getRadioButtonValue(fieldName) {
    var rbVal = $('#' + fieldName).find("input[type='radio']:checked").val();
    return rbVal;
}

function setRadioButtonValue(fieldName, rbVal) {

    var varCheckedVal;

    if (rbVal === true) {
        varCheckedVal = 1;
    }
    else if (rbVal === false) {
        varCheckedVal = 0;
    }
    else {
        varCheckedVal = rbVal;
    }
    var radioButton = $("#" + fieldName).find("input[value=" + varCheckedVal + "]");
    radioButton.prop("checked", true);
}

function setBooleanRadioButtonValue(fieldName, rbVal) {
    var rbNo = $("#" + fieldName + "_0");
    var rbYes = $("#" + fieldName + "_1");

    if (rbVal == 1 || rbVal == true) {
        rbNo.prop("checked", false);
        rbYes.prop("checked", true);    
    }
    else {
        rbNo.prop("checked", true);
        rbYes.prop("checked", false);    
    }
}

// Function to hide fields:
// - hides the field with the given name
// - makes it not required so no validation errors are shown
function hideField(fieldName) {
    var field = $("#" + fieldName);
    field.closest("td").hide();

    makeNotRequired(fieldName);
}

// Function to reset the value of the field.
// Note the special handling for date fields
function resetField(fieldName) {
    var field = $("#" + fieldName);
    var datePicker = field.next().data("DateTimePicker");
    var booleanOrChoiceField=field.hasClass('picklist') || field.hasClass('boolean-radio');

    // if the field is a date field
    if (typeof(datePicker) != "undefined"){

        var dateField = $("#" + fieldName + "_datepicker_description");
        dateField.val("");
    }
    else if(booleanOrChoiceField){
        var noOfOptionSetInputs= $(field).find("input").length;
        for(var i=0;i<noOfOptionSetInputs;i++)
        {
            $('input[id="'+fieldName+'_'+i+'"]').prop('checked', false);
        }
    }
    else{
        field.val("");
    }
}

// Function to hide and reset (clear) fields:
// - hides the field with the given name
// - makes it not required so no validation errors are shown
// - resets its value 
function hideAndResetField(fieldName) {
    hideField(fieldName);
    resetField(fieldName);
}

// Function to show fields
function showField(fieldName) {
    // $("#" + fieldName).closest("td").find("div.control, div.info").show();
    $("#" + fieldName).closest("td").show();
}

// Function to hide section
function hideSection(sectionName) {
    $("table[data-name='" + sectionName + "']").parent().hide();
}
    
// Function to show section
function showSection(sectionName) {
    $("table[data-name='" + sectionName + "']").parent().show();
}

function resizeTextAreaField(fieldName) {
    $('#' + fieldName).css('height', $('#' + fieldName).prop('scrollHeight'));
}

function showOptionElement(optionElem) {
   optionElem.next().css({ "display": "block" });
   optionElem.css({ "display": "inline-block" });   
}

function hideOptionElement(optionElem) {
   optionElem.next().css({ "display": "none" });
   optionElem.css({ "display": "none" });
}

function showOptionValue(fieldName, optionValue) {
    var fieldInputControl = $("#" + fieldName).find("input[value='" + optionValue + "']");
    var fieldLabel = fieldInputControl.next();

    // Show the radio button, its label and the <br> next to the label
    fieldInputControl.css({ "display": "inline-block" });
    fieldLabel.css({ "display": "inline-block" });
    fieldLabel.next().css({ "display": "block" });
}

function hideOptionValue(fieldName, optionValue) {
    var fieldInputControl = $("#" + fieldName).find("input[value='" + optionValue + "']");
    var fieldLabel = fieldInputControl.next();

    // Hide the radio button, its label and the <br> next to the label
    fieldInputControl.css({ "display": "none" });
    fieldLabel.css({ "display": "none" });
    fieldLabel.next().css({ "display": "none" });
}

// Function to hide tab
function hideTab(tabName) {
    $("div[data-name='" + tabName + "']").hide();
    $(".tab[data-name='" + tabName + "']").prev().hide();
}
    
// Function to show tab
function showTab(tabName) {
    $("div[data-name='" + tabName + "']").show();
    $(".tab[data-name='" + tabName + "']").prev().show();
}

function prependCurrencySymbolToCurrencyField(fieldName) {
    var fieldID = "#" + fieldName;
    var field = $(fieldID);

    // If field is set to read only on the form in CRM, don't prepend the currency symbol
    if (field.hasClass("readonly")) {
        return;
    }

    if ($("span.input-group-addon +input" + fieldID).length === 0) {
        $('<div class="input-group">').appendTo(field.parent()).append('<span class="input-group-addon">$</span>').append(field);
    }
}
// function prependPercentageSymbolToField(fieldName) {
//     var fieldID = "#" + fieldName;
//     var field = $(fieldID);

//     // If the field is set to read-only on the form, don't prepend the percentage symbol
//     if (field.hasClass("readonly")) {
//         return;
//     }

//     // Check if the percentage symbol is not already present
//     if ($("span.input-group-addon +input" + fieldID).length === 0) {
//         // Create an input-group structure with a percentage symbol and append it to the parent of the field
//         $('<div class="input-group">').appendTo(field.parent()).append('<span class="input-group-addon">%</span>').append(field);
//     }
// }

function getCurrencyFieldValueAsFloat(fieldName) {
    var currencyfieldValue = $("#" + fieldName).val();
    return getCurrencyValueAsFloat(currencyfieldValue);
}

function setCurrencyFieldValueToNumericAmount(fieldName, numericAmount) {
    var formattedAmount = numericAmount.toString().replace(/\d(?=(?:\d{3})+$)/g, '$&,'); 
    $("#" + fieldName).val(formattedAmount );
}

function getCurrencyValueAsFloat(currencyValue) {
    var currencyValueNumericOnly = currencyValue.replace(/[^0-9.-]+/g, "");
    var floatValue = (currencyValueNumericOnly.length > 0 ? parseFloat(currencyValueNumericOnly) : 0);
    return floatValue;    
}

function hideApplicationWithdrawalFields(){
    var status=$('#statuscode_EntityStatus').val();
    if(status==='2'){// Withdrawn by Applicant
        var withdrawalReason=$('#nhfic_withdrawalreason').val();
        showField('nhfic_withdrawalreason');
        showField('nhfic_withdrawaldate');
        if(withdrawalReason==='991900099'){// if reason is others
            showField('nhfic_otherwithdrawalreason');
            resizeTextAreaField("nhfic_otherwithdrawalreason");
        }else{
            hideField('nhfic_otherwithdrawalreason');
        }
    }else{
        hideField('nhfic_withdrawaldate');
        hideField('nhfic_withdrawalreason');
        hideField('nhfic_otherwithdrawalreason');
    }
}